#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This script is licensed under GNU GPL version 2.0 or above
# (c) 2024 Antonio J. Delgado
"""Ensure OVH DNS records"""

import sys
import os
import logging
from logging.handlers import SysLogHandler
import click
import click_config_file
import ovh


class OvhDnsEnsure:
    '''Ensure DNS records in OVH'''

    def __init__(self, **kwargs):
        self.config = kwargs
        if 'log_file' not in kwargs or kwargs['log_file'] is None:
            self.config['log_file'] = os.path.join(
                os.environ.get(
                    'HOME',
                    os.environ.get(
                        'USERPROFILE',
                        os.getcwd()
                    )
                ),
                'log',
                'ovh_dns_ensure.log'
            )
        self._init_log()
        self._ovh_login()
        self._get_current_state()
        if self.config['state'] == 'absent':
            self._ensure_absent()
        else:
            self._ensure_present()

    def _ensure_absent(self):
        if self.current_state == 'absent':
            return True
        else:
            for record in self.records:
                self._log.debug(
                    'Deleting record %s',
                    record
                )
                if not self.config['dummy']:
                    result = self.ovh.delete(f"/domain/zone/{self.config['zone']}/record/{record}")
                    self._log.debug(result)

    def _ensure_present(self):
        if self.current_state == 'same':
            return True
        elif self.current_state == 'absent':
            self._log.debug(
                'Creating new record'
            )
            if not self.config['dummy']:
                result = self.ovh.post(f"/domain/zone/{self.config['zone']}/record",
                    ttl = self.config['ttl'],
                    fieldType = self.config['type'],
                    subDomain = self.config['subdomain'],
                    target = self.config['target'],
                )
                self._log.debug(result)
        elif self.current_state == 'different':
            for record in self.records:
                if not self.config['dummy']:
                    self._log.debug(
                        'Deleting record %s',
                        record
                    )
                    result = self.ovh.delete(f"/domain/zone/{self.config['zone']}/record/{record}")
                    self._log.debug(result)
            self._log.debug(
                "Creating new record..."
            )
            if not self.config['dummy']:
                result = self.ovh.post(f"/domain/zone/{self.config['zone']}/record/",
                    subDomain = self.config['subdomain'],
                    fieldType = self.config['type'],
                    target = self.config['target'],
                    ttl = self.config['ttl'],
                )
                self._log.debug(result)


    def _get_current_state(self):
        params = {
            # "fieldType": self.config['type'],
            "subDomain": self.config['subdomain']
        }
        self._log.debug(
            "Getting all records with subdomain '%s'...",
            # self.config['type'],
            self.config['subdomain']
        )
        self.records = self.ovh.get(f"/domain/zone/{self.config['zone']}/record", **params)
        self.current_state = 'absent'
        for record in self.records:
            self._log.debug(
                "Checking record: %s...",
                record
            )
            data = self.ovh.get(f"/domain/zone/{self.config['zone']}/record/{record}")
            if data['target'] == self.config['target'] and data['ttl'] == self.config['ttl']:
                self.current_state = 'same'
                self._log.debug(
                    'A record with the same type, TTL and target exists. %s',
                    data
                )
        if self.current_state == 'absent' and len(self.records) == 0:
            self._log.debug(
                "Doesn't exist a record with that type and target"
            )
        elif self.current_state == 'absent' and len(self.records) > 0:
            self._log.debug(
                "There are %s records with same type but different target or TTL",
                len(self.records)
            )
            self.current_state = 'different'
        elif self.current_state == 'absent':
            self._log.debug(
                "There are no records with same type but different target or TTL"
            )

    def _ovh_login(self):
        self.ovh = ovh.Client(
            endpoint=self.config['ovh_endpoint'],
            application_key=self.config['ovh_app_key'],
            application_secret=self.config['ovh_app_secret'],
            consumer_key=self.config['ovh_consumer_key'],
        )

    def _init_log(self):
        ''' Initialize log object '''
        self._log = logging.getLogger("ovh_dns_ensure")
        self._log.setLevel(logging.DEBUG)

        sysloghandler = SysLogHandler()
        sysloghandler.setLevel(logging.DEBUG)
        self._log.addHandler(sysloghandler)

        streamhandler = logging.StreamHandler(sys.stdout)
        streamhandler.setLevel(
            logging.getLevelName(self.config.get("debug_level", 'INFO'))
        )
        self._log.addHandler(streamhandler)

        if 'log_file' in self.config:
            log_file = self.config['log_file']
        else:
            home_folder = os.environ.get(
                'HOME', os.environ.get('USERPROFILE', '')
            )
            log_folder = os.path.join(home_folder, "log")
            log_file = os.path.join(log_folder, "ovh_dns_ensure.log")

        if not os.path.exists(os.path.dirname(log_file)):
            os.mkdir(os.path.dirname(log_file))

        filehandler = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=102400000
        )
        # create formatter
        formatter = logging.Formatter(
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'
        )
        filehandler.setFormatter(formatter)
        filehandler.setLevel(logging.DEBUG)
        self._log.addHandler(filehandler)
        return True


@click.command()
@click.option(
    "--debug-level",
    "-d",
    default="INFO",
    type=click.Choice(
        ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG", "NOTSET"],
        case_sensitive=False,
    ),
    help='Set the debug level for the standard output.'
)
@click.option('--log-file', '-l', help="File to store all debug messages.")
@click.option(
    "--dummy","-n",
    is_flag=True,
    help="Don't do anything, just show what would be done."
)
@click.option(
    '--zone', '-z',
    required=True,
    help='DNS zone to do the changes'
)
@click.option(
    '--subdomain',
    '-s',
    required=True,
    help='Subdomain to ensure'
)
@click.option(
    '--type',
    '-T',
    required=True,
    default='A',
    type=click.Choice(
        ["A", "AAAA", "CAA", "CNAME", "DKIM", "DMARC", "DNAME", "LOC", "MX", "NAPTR", "NS", "PTR", "SPF", "SRV", "SSHFP", "TLSA", "TXT"],
        case_sensitive=False,
    ),
    help='DNS record type'
)
@click.option(
    '--target',
    '-t',
    required=True,
    help='Desired target for the subdomain'
)
@click.option(
    '--ttl',
    '-l',
    default=0,
    required=True,
    help='Desired target Time-To-Live'
)
@click.option(
    '--state',
    '-S',
    default='present',
    type=click.Choice(
        ["present", "absent"],
        case_sensitive=False,
    ),
    help='State to ensure.'
)
@click.option(
    '--ovh-app-key',
    '-a',
    required=True,
    help='OVH Application key. Better use the configuration file for safety.'
)
@click.option(
    '--ovh-app-secret',
    '-e',
    required=True,
    help='OVH Application secret. Better use the configuration file for safety.'
)
@click.option(
    '--ovh-consumer-key',
    '-c',
    required=True,
    help='OVH Consumer key. Better use the configuration file for safety.'
)
@click.option(
    '--ovh-endpoint',
    '-E',
    required=True,
    default='ovh-eu',
    type=click.Choice(
        [
            "ovh-eu", "ovh-us", 'ovh-ca',
            'soyoustart-eu', 'soyoustart-ca',
            'kimsufi-eu', 'kimsufi-ca'
        ],
        case_sensitive=False,
    ),
    help='OVH Server to use.'
)
@click_config_file.configuration_option()
def __main__(**kwargs):
    return OvhDnsEnsure(**kwargs)


if __name__ == "__main__":
    __main__()
